@@ -89,19 +89,18 @@ unsafe fn horiz_convolution_8u4x(
     let initial = _mm256_set1_epi32(1 << (precision - 1));
 
     #[rustfmt::skip]
-        let sh1 = _mm256_set_epi8(
+    let sh1 = _mm256_set_epi8(
         -1, 7, -1, 3, -1, 6, -1, 2, -1, 5, -1, 1, -1, 4, -1, 0,
         -1, 7, -1, 3, -1, 6, -1, 2, -1, 5, -1, 1, -1, 4, -1, 0,
     );
     #[rustfmt::skip]
-        let sh2 = _mm256_set_epi8(
+    let sh2 = _mm256_set_epi8(
         -1, 15, -1, 11, -1, 14, -1, 10, -1, 13, -1, 9, -1, 12, -1, 8,
         -1, 15, -1, 11, -1, 14, -1, 10, -1, 13, -1, 9, -1, 12, -1, 8,
     );
 
     for (dst_x, coeffs_chunk) in coefficients_chunks.iter().enumerate() {
-        let x_start = coeffs_chunk.start as usize;
-        let mut x: usize = 0;
+        let mut x = coeffs_chunk.start as usize;
 
         let mut sss0 = initial;
         let mut sss1 = initial;
@@ -114,22 +113,22 @@ unsafe fn horiz_convolution_8u4x(
             let mmk0 = simd_utils::ptr_i16_to_256set1_epi32(k, 0);
             let mmk1 = simd_utils::ptr_i16_to_256set1_epi32(k, 2);
 
-            let mut source = _mm256_inserti128_si256::<1>(
-                _mm256_castsi128_si256(simd_utils::loadu_si128(s_row0, x + x_start)),
-                simd_utils::loadu_si128(s_row1, x + x_start),
+            let source = _mm256_inserti128_si256::<1>(
+                _mm256_castsi128_si256(simd_utils::loadu_si128(s_row0, x)),
+                simd_utils::loadu_si128(s_row1, x),
             );
-            let mut pix = _mm256_shuffle_epi8(source, sh1);
+            let pix = _mm256_shuffle_epi8(source, sh1);
             sss0 = _mm256_add_epi32(sss0, _mm256_madd_epi16(pix, mmk0));
-            pix = _mm256_shuffle_epi8(source, sh2);
+            let pix = _mm256_shuffle_epi8(source, sh2);
             sss0 = _mm256_add_epi32(sss0, _mm256_madd_epi16(pix, mmk1));
 
-            source = _mm256_inserti128_si256::<1>(
-                _mm256_castsi128_si256(simd_utils::loadu_si128(s_row2, x + x_start)),
-                simd_utils::loadu_si128(s_row3, x + x_start),
+            let source = _mm256_inserti128_si256::<1>(
+                _mm256_castsi128_si256(simd_utils::loadu_si128(s_row2, x)),
+                simd_utils::loadu_si128(s_row3, x),
             );
-            pix = _mm256_shuffle_epi8(source, sh1);
+            let pix = _mm256_shuffle_epi8(source, sh1);
             sss1 = _mm256_add_epi32(sss1, _mm256_madd_epi16(pix, mmk0));
-            pix = _mm256_shuffle_epi8(source, sh2);
+            let pix = _mm256_shuffle_epi8(source, sh2);
             sss1 = _mm256_add_epi32(sss1, _mm256_madd_epi16(pix, mmk1));
 
             x += 4;
@@ -141,40 +140,39 @@ unsafe fn horiz_convolution_8u4x(
         for k in coeffs_by_2 {
             let mmk = simd_utils::ptr_i16_to_256set1_epi32(k, 0);
 
-            let mut pix = _mm256_inserti128_si256::<1>(
-                _mm256_castsi128_si256(simd_utils::loadl_epi64(s_row0, x + x_start)),
-                simd_utils::loadl_epi64(s_row1, x + x_start),
+            let source = _mm256_inserti128_si256::<1>(
+                _mm256_castsi128_si256(simd_utils::loadl_epi64(s_row0, x)),
+                simd_utils::loadl_epi64(s_row1, x),
             );
-            pix = _mm256_shuffle_epi8(pix, sh1);
+            let pix = _mm256_shuffle_epi8(source, sh1);
             sss0 = _mm256_add_epi32(sss0, _mm256_madd_epi16(pix, mmk));
 
-            pix = _mm256_inserti128_si256::<1>(
-                _mm256_castsi128_si256(simd_utils::loadl_epi64(s_row2, x + x_start)),
-                simd_utils::loadl_epi64(s_row3, x + x_start),
+            let source = _mm256_inserti128_si256::<1>(
+                _mm256_castsi128_si256(simd_utils::loadl_epi64(s_row2, x)),
+                simd_utils::loadl_epi64(s_row3, x),
             );
-            pix = _mm256_shuffle_epi8(pix, sh1);
+            let pix = _mm256_shuffle_epi8(source, sh1);
             sss1 = _mm256_add_epi32(sss1, _mm256_madd_epi16(pix, mmk));
 
             x += 2;
         }
 
-        for &k in reminder2 {
+        if let Some(&k) = reminder2.get(0) {
             // [16] xx k0 xx k0 xx k0 xx k0 xx k0 xx k0 xx k0 xx k0
             let mmk = _mm256_set1_epi32(k as i32);
 
             // [16] xx a0 xx b0 xx g0 xx r0 xx a0 xx b0 xx g0 xx r0
-            let mut pix = _mm256_inserti128_si256::<1>(
-                _mm256_castsi128_si256(simd_utils::mm_cvtepu8_epi32(s_row0, x + x_start)),
-                simd_utils::mm_cvtepu8_epi32(s_row1, x + x_start),
+            let pix = _mm256_inserti128_si256::<1>(
+                _mm256_castsi128_si256(simd_utils::mm_cvtepu8_epi32(s_row0, x)),
+                simd_utils::mm_cvtepu8_epi32(s_row1, x),
             );
             sss0 = _mm256_add_epi32(sss0, _mm256_madd_epi16(pix, mmk));
 
-            pix = _mm256_inserti128_si256::<1>(
-                _mm256_castsi128_si256(simd_utils::mm_cvtepu8_epi32(s_row2, x + x_start)),
-                simd_utils::mm_cvtepu8_epi32(s_row3, x + x_start),
+            let pix = _mm256_inserti128_si256::<1>(
+                _mm256_castsi128_si256(simd_utils::mm_cvtepu8_epi32(s_row2, x)),
+                simd_utils::mm_cvtepu8_epi32(s_row3, x),
             );
             sss1 = _mm256_add_epi32(sss1, _mm256_madd_epi16(pix, mmk));
-            x += 1;
         }
 
         macro_rules! call {
@@ -214,43 +212,42 @@ unsafe fn horiz_convolution_8u(
     precision: u8,
 ) {
     #[rustfmt::skip]
-        let sh1 = _mm256_set_epi8(
+    let sh1 = _mm256_set_epi8(
         -1, 7, -1, 3, -1, 6, -1, 2, -1, 5, -1, 1, -1, 4, -1, 0,
         -1, 7, -1, 3, -1, 6, -1, 2, -1, 5, -1, 1, -1, 4, -1, 0,
     );
     #[rustfmt::skip]
-        let sh2 = _mm256_set_epi8(
+    let sh2 = _mm256_set_epi8(
         11, 10, 9, 8, 11, 10, 9, 8, 11, 10, 9, 8, 11, 10, 9, 8,
         3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0,
     );
     #[rustfmt::skip]
-        let sh3 = _mm256_set_epi8(
+    let sh3 = _mm256_set_epi8(
         -1, 15, -1, 11, -1, 14, -1, 10, -1, 13, -1, 9, -1, 12, -1, 8,
         -1, 15, -1, 11, -1, 14, -1, 10, -1, 13, -1, 9, -1, 12, -1, 8,
     );
     #[rustfmt::skip]
-        let sh4 = _mm256_set_epi8(
+    let sh4 = _mm256_set_epi8(
         15, 14, 13, 12, 15, 14, 13, 12, 15, 14, 13, 12, 15, 14, 13, 12,
         7, 6, 5, 4, 7, 6, 5, 4, 7, 6, 5, 4, 7, 6, 5, 4,
     );
     #[rustfmt::skip]
-        let sh5 = _mm256_set_epi8(
+    let sh5 = _mm256_set_epi8(
         -1, 15, -1, 11, -1, 14, -1, 10, -1, 13, -1, 9, -1, 12, -1, 8,
         -1, 7, -1, 3, -1, 6, -1, 2, -1, 5, -1, 1, -1, 4, -1, 0,
     );
     #[rustfmt::skip]
-        let sh6 = _mm256_set_epi8(
+    let sh6 = _mm256_set_epi8(
         7, 6, 5, 4, 7, 6, 5, 4, 7, 6, 5, 4, 7, 6, 5, 4,
         3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0,
     );
     let sh7 = _mm_set_epi8(-1, 7, -1, 3, -1, 6, -1, 2, -1, 5, -1, 1, -1, 4, -1, 0);
 
     for (dst_x, &coeffs_chunk) in coefficients_chunks.iter().enumerate() {
-        let x_start = coeffs_chunk.start as usize;
-        let mut x: usize = 0;
+        let mut x = coeffs_chunk.start as usize;
         let mut coeffs = coeffs_chunk.values;
 
-        let mut sss: __m128i = if coeffs.len() < 8 {
+        let mut sss = if coeffs.len() < 8 {
             _mm_set1_epi32(1 << (precision - 1))
         } else {
             // Lower part will be added to higher, use only half of the error
@@ -263,14 +260,14 @@ unsafe fn horiz_convolution_8u(
                 let tmp = simd_utils::loadu_si128(k, 0);
                 let ksource = _mm256_insertf128_si256::<1>(_mm256_castsi128_si256(tmp), tmp);
 
-                let source = simd_utils::loadu_si256(src_row, x + x_start);
+                let source = simd_utils::loadu_si256(src_row, x);
 
-                let mut pix = _mm256_shuffle_epi8(source, sh1);
-                let mut mmk = _mm256_shuffle_epi8(ksource, sh2);
+                let pix = _mm256_shuffle_epi8(source, sh1);
+                let mmk = _mm256_shuffle_epi8(ksource, sh2);
                 sss256 = _mm256_add_epi32(sss256, _mm256_madd_epi16(pix, mmk));
 
-                pix = _mm256_shuffle_epi8(source, sh3);
-                mmk = _mm256_shuffle_epi8(ksource, sh4);
+                let pix = _mm256_shuffle_epi8(source, sh3);
+                let mmk = _mm256_shuffle_epi8(ksource, sh4);
                 sss256 = _mm256_add_epi32(sss256, _mm256_madd_epi16(pix, mmk));
 
                 x += 8;
@@ -283,7 +280,7 @@ unsafe fn horiz_convolution_8u(
                 let tmp = simd_utils::loadl_epi64(k, 0);
                 let ksource = _mm256_insertf128_si256::<1>(_mm256_castsi128_si256(tmp), tmp);
 
-                let tmp = simd_utils::loadu_si128(src_row, x + x_start);
+                let tmp = simd_utils::loadu_si128(src_row, x);
                 let source = _mm256_insertf128_si256::<1>(_mm256_castsi128_si256(tmp), tmp);
 
                 let pix = _mm256_shuffle_epi8(source, sh5);
@@ -304,19 +301,17 @@ unsafe fn horiz_convolution_8u(
 
         for k in coeffs_by_2 {
             let mmk = simd_utils::ptr_i16_to_set1_epi32(k, 0);
-            let source = simd_utils::loadl_epi64(src_row, x + x_start);
+            let source = simd_utils::loadl_epi64(src_row, x);
             let pix = _mm_shuffle_epi8(source, sh7);
             sss = _mm_add_epi32(sss, _mm_madd_epi16(pix, mmk));
 
             x += 2
         }
 
-        for &k in reminder1 {
-            let pix = simd_utils::mm_cvtepu8_epi32(src_row, x + x_start);
+        if let Some(&k) = reminder1.get(0) {
+            let pix = simd_utils::mm_cvtepu8_epi32(src_row, x);
             let mmk = _mm_set1_epi32(k as i32);
             sss = _mm_add_epi32(sss, _mm_madd_epi16(pix, mmk));
-
-            x += 1;
         }
 
         macro_rules! call {
@@ -364,16 +359,16 @@ unsafe fn vert_convolution_8u(
             let source1 = simd_utils::loadu_si256(s_row1, x); // top line
             let source2 = simd_utils::loadu_si256(s_row2, x); // bottom line
 
-            let mut source = _mm256_unpacklo_epi8(source1, source2);
-            let mut pix = _mm256_unpacklo_epi8(source, _mm256_setzero_si256());
+            let source = _mm256_unpacklo_epi8(source1, source2);
+            let pix = _mm256_unpacklo_epi8(source, _mm256_setzero_si256());
             sss0 = _mm256_add_epi32(sss0, _mm256_madd_epi16(pix, mmk));
-            pix = _mm256_unpackhi_epi8(source, _mm256_setzero_si256());
+            let pix = _mm256_unpackhi_epi8(source, _mm256_setzero_si256());
             sss1 = _mm256_add_epi32(sss1, _mm256_madd_epi16(pix, mmk));
 
-            source = _mm256_unpackhi_epi8(source1, source2);
-            pix = _mm256_unpacklo_epi8(source, _mm256_setzero_si256());
+            let source = _mm256_unpackhi_epi8(source1, source2);
+            let pix = _mm256_unpacklo_epi8(source, _mm256_setzero_si256());
             sss2 = _mm256_add_epi32(sss2, _mm256_madd_epi16(pix, mmk));
-            pix = _mm256_unpackhi_epi8(source, _mm256_setzero_si256());
+            let pix = _mm256_unpackhi_epi8(source, _mm256_setzero_si256());
             sss3 = _mm256_add_epi32(sss3, _mm256_madd_epi16(pix, mmk));
 
             y += 2;
@@ -385,16 +380,16 @@ unsafe fn vert_convolution_8u(
             let source1 = simd_utils::loadu_si256(s_row, x); // top line
             let source2 = _mm256_setzero_si256(); // bottom line is empty
 
-            let mut source = _mm256_unpacklo_epi8(source1, source2);
-            let mut pix = _mm256_unpacklo_epi8(source, _mm256_setzero_si256());
+            let source = _mm256_unpacklo_epi8(source1, source2);
+            let pix = _mm256_unpacklo_epi8(source, _mm256_setzero_si256());
             sss0 = _mm256_add_epi32(sss0, _mm256_madd_epi16(pix, mmk));
-            pix = _mm256_unpackhi_epi8(source, _mm256_setzero_si256());
+            let pix = _mm256_unpackhi_epi8(source, _mm256_setzero_si256());
             sss1 = _mm256_add_epi32(sss1, _mm256_madd_epi16(pix, mmk));
 
-            source = _mm256_unpackhi_epi8(source1, _mm256_setzero_si256());
-            pix = _mm256_unpacklo_epi8(source, _mm256_setzero_si256());
+            let source = _mm256_unpackhi_epi8(source1, _mm256_setzero_si256());
+            let pix = _mm256_unpacklo_epi8(source, _mm256_setzero_si256());
             sss2 = _mm256_add_epi32(sss2, _mm256_madd_epi16(pix, mmk));
-            pix = _mm256_unpackhi_epi8(source, _mm256_setzero_si256());
+            let pix = _mm256_unpackhi_epi8(source, _mm256_setzero_si256());
             sss3 = _mm256_add_epi32(sss3, _mm256_madd_epi16(pix, mmk));
         }
 
@@ -430,9 +425,9 @@ unsafe fn vert_convolution_8u(
             let source2 = simd_utils::loadl_epi64(s_row2, x); // bottom line
 
             let source = _mm_unpacklo_epi8(source1, source2);
-            let mut pix = _mm_unpacklo_epi8(source, _mm_setzero_si128());
+            let pix = _mm_unpacklo_epi8(source, _mm_setzero_si128());
             sss0 = _mm_add_epi32(sss0, _mm_madd_epi16(pix, mmk));
-            pix = _mm_unpackhi_epi8(source, _mm_setzero_si128());
+            let pix = _mm_unpackhi_epi8(source, _mm_setzero_si128());
             sss1 = _mm_add_epi32(sss1, _mm_madd_epi16(pix, mmk));
 
             y += 2;
@@ -445,9 +440,9 @@ unsafe fn vert_convolution_8u(
             let source2 = _mm_setzero_si128(); // bottom line is empty
 
             let source = _mm_unpacklo_epi8(source1, source2);
-            let mut pix = _mm_unpacklo_epi8(source, _mm_setzero_si128());
+            let pix = _mm_unpacklo_epi8(source, _mm_setzero_si128());
             sss0 = _mm_add_epi32(sss0, _mm_madd_epi16(pix, mmk));
-            pix = _mm_unpackhi_epi8(source, _mm_setzero_si128());
+            let pix = _mm_unpackhi_epi8(source, _mm_setzero_si128());
             sss1 = _mm_add_epi32(sss1, _mm_madd_epi16(pix, mmk));
         }
 
