@@ -1,15 +1,13 @@
-use std::fs::File;
 use std::num::NonZeroU32;
 
-use image::codecs::png::PngEncoder;
-use image::io::Reader as ImageReader;
-use image::{ColorType, DynamicImage, GenericImageView};
-
 use fast_image_resize::pixels::*;
 use fast_image_resize::{
     CpuExtensions, DifferentTypesOfPixelsError, FilterType, Image, ImageView, PixelType, ResizeAlg,
     Resizer,
 };
+use utils::{cpu_ext_into_str, PixelExt};
+
+mod utils;
 
 fn get_new_height(src_image: &ImageView, new_width: u32) -> u32 {
     let scale = new_width as f32 / src_image.width().get() as f32;
@@ -19,36 +17,6 @@ fn get_new_height(src_image: &ImageView, new_width: u32) -> u32 {
 const NEW_WIDTH: u32 = 255;
 const NEW_BIG_WIDTH: u32 = 5016;
 
-fn save_result(image: &Image, name: &str) {
-    if std::env::var("DONT_SAVE_RESULT").unwrap_or_else(|_| "".to_owned()) == "1" {
-        return;
-    }
-    std::fs::create_dir_all("./data/result").unwrap();
-    let mut file = File::create(format!("./data/result/{}.png", name)).unwrap();
-    let color_type = match image.pixel_type() {
-        PixelType::U8x3 => ColorType::Rgb8,
-        PixelType::U8x4 => ColorType::Rgba8,
-        PixelType::U8 => ColorType::L8,
-        _ => panic!("Unsupported type of pixels"),
-    };
-    PngEncoder::new(&mut file)
-        .encode(
-            image.buffer(),
-            image.width().get(),
-            image.height().get(),
-            color_type,
-        )
-        .unwrap();
-}
-
-fn image_checksum<const N: usize>(buffer: &[u8]) -> [u32; N] {
-    let mut res = [0u32; N];
-    for pixel in buffer.chunks_exact(N) {
-        res.iter_mut().zip(pixel).for_each(|(d, &s)| *d += s as u32);
-    }
-    res
-}
-
 #[test]
 fn try_resize_to_other_pixel_type() {
     let src_image = U8x4::load_big_src_image();
@@ -64,88 +32,6 @@ fn try_resize_to_other_pixel_type() {
     ));
 }
 
-trait PixelExt: Pixel {
-    fn pixel_type_str() -> &'static str {
-        match Self::pixel_type() {
-            PixelType::U8 => "u8",
-            PixelType::U8x3 => "u8x3",
-            PixelType::U8x4 => "u8x4",
-            PixelType::I32 => "i32",
-            PixelType::F32 => "f32",
-        }
-    }
-
-    fn load_big_src_image() -> Image<'static> {
-        let img = ImageReader::open("./data/nasa-4928x3279.png")
-            .unwrap()
-            .decode()
-            .unwrap();
-        Image::from_vec_u8(
-            NonZeroU32::new(img.width()).unwrap(),
-            NonZeroU32::new(img.height()).unwrap(),
-            Self::img_into_bytes(img),
-            Self::pixel_type(),
-        )
-        .unwrap()
-    }
-
-    fn load_small_src_image() -> Image<'static> {
-        let img = ImageReader::open("./data/nasa-852x567.png")
-            .unwrap()
-            .decode()
-            .unwrap();
-        Image::from_vec_u8(
-            NonZeroU32::new(img.width()).unwrap(),
-            NonZeroU32::new(img.height()).unwrap(),
-            Self::img_into_bytes(img),
-            Self::pixel_type(),
-        )
-        .unwrap()
-    }
-
-    fn img_into_bytes(img: DynamicImage) -> Vec<u8>;
-}
-
-impl PixelExt for U8 {
-    fn img_into_bytes(img: DynamicImage) -> Vec<u8> {
-        img.to_luma8().into_raw()
-    }
-}
-
-impl PixelExt for U8x3 {
-    fn img_into_bytes(img: DynamicImage) -> Vec<u8> {
-        img.to_rgb8().into_raw()
-    }
-}
-
-impl PixelExt for U8x4 {
-    fn img_into_bytes(img: DynamicImage) -> Vec<u8> {
-        img.to_rgba8().into_raw()
-    }
-}
-
-impl PixelExt for I32 {
-    fn img_into_bytes(img: DynamicImage) -> Vec<u8> {
-        img.to_luma16()
-            .as_raw()
-            .iter()
-            .map(|&p| p as u32 * (i16::MAX as u32 + 1))
-            .flat_map(|val| val.to_le_bytes())
-            .collect()
-    }
-}
-
-impl PixelExt for F32 {
-    fn img_into_bytes(img: DynamicImage) -> Vec<u8> {
-        img.to_luma16()
-            .as_raw()
-            .iter()
-            .map(|&p| p as f32 * (i16::MAX as f32 + 1.0))
-            .flat_map(|val| val.to_le_bytes())
-            .collect()
-    }
-}
-
 fn downscale_test<P: PixelExt>(resize_alg: ResizeAlg, cpu_extensions: CpuExtensions) -> Vec<u8> {
     let image = P::load_big_src_image();
     assert_eq!(image.pixel_type(), P::pixel_type());
@@ -179,23 +65,13 @@ fn downscale_test<P: PixelExt>(resize_alg: ResizeAlg, cpu_extensions: CpuExtensi
         _ => "unknown",
     };
 
-    let ext_name = match cpu_extensions {
-        CpuExtensions::None => "native",
-        #[cfg(target_arch = "x86_64")]
-        CpuExtensions::Sse2 => "sse2",
-        #[cfg(target_arch = "x86_64")]
-        CpuExtensions::Sse4_1 => "sse41",
-        #[cfg(target_arch = "x86_64")]
-        CpuExtensions::Avx2 => "avx2",
-    };
-
     let name = format!(
         "downscale-{}-{}-{}",
         P::pixel_type_str(),
         alg_name,
-        ext_name
+        cpu_ext_into_str(cpu_extensions),
     );
-    save_result(&result, &name);
+    utils::save_result(&result, &name);
     result.buffer().to_owned()
 }
 
@@ -232,18 +108,13 @@ fn upscale_test<P: PixelExt>(resize_alg: ResizeAlg, cpu_extensions: CpuExtension
         _ => "unknown",
     };
 
-    let ext_name = match cpu_extensions {
-        CpuExtensions::None => "native",
-        #[cfg(target_arch = "x86_64")]
-        CpuExtensions::Sse2 => "sse2",
-        #[cfg(target_arch = "x86_64")]
-        CpuExtensions::Sse4_1 => "sse41",
-        #[cfg(target_arch = "x86_64")]
-        CpuExtensions::Avx2 => "avx2",
-    };
-
-    let name = format!("upscale-{}-{}-{}", P::pixel_type_str(), alg_name, ext_name);
-    save_result(&result, &name);
+    let name = format!(
+        "upscale-{}-{}-{}",
+        P::pixel_type_str(),
+        alg_name,
+        cpu_ext_into_str(cpu_extensions),
+    );
+    utils::save_result(&result, &name);
     result.buffer().to_owned()
 }
 
@@ -251,7 +122,7 @@ fn upscale_test<P: PixelExt>(resize_alg: ResizeAlg, cpu_extensions: CpuExtension
 fn downscale_u8() {
     type P = U8;
     let buffer = downscale_test::<P>(ResizeAlg::Nearest, CpuExtensions::None);
-    assert_eq!(image_checksum::<1>(&buffer), [2920317]);
+    assert_eq!(utils::image_checksum::<1>(&buffer), [2920317]);
 
     let mut cpu_extensions_vec = vec![CpuExtensions::None];
     #[cfg(target_arch = "x86_64")]
@@ -261,7 +132,7 @@ fn downscale_u8() {
     for cpu_extensions in cpu_extensions_vec {
         let buffer =
             downscale_test::<P>(ResizeAlg::Convolution(FilterType::Lanczos3), cpu_extensions);
-        assert_eq!(image_checksum::<1>(&buffer), [2923520]);
+        assert_eq!(utils::image_checksum::<1>(&buffer), [2923520]);
     }
 }
 
@@ -269,7 +140,7 @@ fn downscale_u8() {
 fn upscale_u8() {
     type P = U8;
     let buffer = upscale_test::<P>(ResizeAlg::Nearest, CpuExtensions::None);
-    assert_eq!(image_checksum::<1>(&buffer), [1148750539]);
+    assert_eq!(utils::image_checksum::<1>(&buffer), [1148750539]);
 
     let mut cpu_extensions_vec = vec![CpuExtensions::None];
     #[cfg(target_arch = "x86_64")]
@@ -279,7 +150,7 @@ fn upscale_u8() {
     for cpu_extensions in cpu_extensions_vec {
         let buffer =
             upscale_test::<P>(ResizeAlg::Convolution(FilterType::Lanczos3), cpu_extensions);
-        assert_eq!(image_checksum::<1>(&buffer), [1148808058]);
+        assert_eq!(utils::image_checksum::<1>(&buffer), [1148808058]);
     }
 }
 
@@ -287,7 +158,10 @@ fn upscale_u8() {
 fn downscale_u8x3() {
     type P = U8x3;
     let buffer = downscale_test::<P>(ResizeAlg::Nearest, CpuExtensions::None);
-    assert_eq!(image_checksum::<3>(&buffer), [2937940, 2945380, 2882679]);
+    assert_eq!(
+        utils::image_checksum::<3>(&buffer),
+        [2937940, 2945380, 2882679]
+    );
 
     let mut cpu_extensions_vec = vec![CpuExtensions::None];
     #[cfg(target_arch = "x86_64")]
@@ -298,7 +172,10 @@ fn downscale_u8x3() {
     for cpu_extensions in cpu_extensions_vec {
         let buffer =
             downscale_test::<P>(ResizeAlg::Convolution(FilterType::Lanczos3), cpu_extensions);
-        assert_eq!(image_checksum::<3>(&buffer), [2942479, 2947850, 2885072]);
+        assert_eq!(
+            utils::image_checksum::<3>(&buffer),
+            [2942479, 2947850, 2885072]
+        );
     }
 }
 
@@ -307,7 +184,7 @@ fn upscale_u8x3() {
     type P = U8x3;
     let buffer = upscale_test::<P>(ResizeAlg::Nearest, CpuExtensions::None);
     assert_eq!(
-        image_checksum::<3>(&buffer),
+        utils::image_checksum::<3>(&buffer),
         [1156008260, 1158417906, 1135087540]
     );
 
@@ -321,7 +198,7 @@ fn upscale_u8x3() {
         let buffer =
             upscale_test::<P>(ResizeAlg::Convolution(FilterType::Lanczos3), cpu_extensions);
         assert_eq!(
-            image_checksum::<3>(&buffer),
+            utils::image_checksum::<3>(&buffer),
             [1156107005, 1158443335, 1135101759]
         );
     }
@@ -332,7 +209,7 @@ fn downscale_u8x4() {
     type P = U8x4;
     let buffer = downscale_test::<P>(ResizeAlg::Nearest, CpuExtensions::None);
     assert_eq!(
-        image_checksum::<4>(&buffer),
+        utils::image_checksum::<4>(&buffer),
         [2937940, 2945380, 2882679, 11054250]
     );
 
@@ -346,7 +223,7 @@ fn downscale_u8x4() {
         let buffer =
             downscale_test::<P>(ResizeAlg::Convolution(FilterType::Lanczos3), cpu_extensions);
         assert_eq!(
-            image_checksum::<4>(&buffer),
+            utils::image_checksum::<4>(&buffer),
             [2942479, 2947850, 2885072, 11054250]
         );
 
@@ -362,7 +239,7 @@ fn upscale_u8x4() {
     type P = U8x4;
     let buffer = upscale_test::<P>(ResizeAlg::Nearest, CpuExtensions::None);
     assert_eq!(
-        image_checksum::<4>(&buffer),
+        utils::image_checksum::<4>(&buffer),
         [1156008260, 1158417906, 1135087540, 4269569040]
     );
 
@@ -376,7 +253,7 @@ fn upscale_u8x4() {
         let buffer =
             upscale_test::<P>(ResizeAlg::Convolution(FilterType::Lanczos3), cpu_extensions);
         assert_eq!(
-            image_checksum::<4>(&buffer),
+            utils::image_checksum::<4>(&buffer),
             [1156107005, 1158443335, 1135101759, 4269569040]
         );
     }
